@@ -12,6 +12,7 @@ use std::time::SystemTime;
 use strum_macros::{Display, EnumString};
 
 use crate::archive::ArchiveMethod;
+use crate::auth::CurrentUser;
 use crate::errors::{self, ContextualError};
 use crate::renderer;
 use percent_encode_sets::PATH_SEGMENT;
@@ -32,6 +33,7 @@ pub struct QueryParameters {
     pub path: Option<PathBuf>,
     pub sort: Option<SortingMethod>,
     pub order: Option<SortingOrder>,
+    pub raw: Option<bool>,
     qrcode: Option<String>,
     download: Option<ArchiveMethod>,
 }
@@ -152,6 +154,9 @@ pub fn directory_listing(
     dir: &actix_files::Directory,
     req: &HttpRequest,
 ) -> io::Result<ServiceResponse> {
+    let extensions = req.extensions();
+    let current_user: Option<&CurrentUser> = extensions.get::<CurrentUser>();
+
     use actix_web::dev::BodyEncoding;
     let conf = req.app_data::<crate::MiniserveConfig>().unwrap();
     let serve_path = req.path();
@@ -374,6 +379,7 @@ pub fn directory_listing(
                         breadcrumbs,
                         &encoded_dir,
                         conf,
+                        current_user,
                     )
                     .into_string(),
                 ),
